package com.persagy.dmp.alarm.jms;

import com.persagy.common.json.JacksonMapper;
import com.persagy.dmp.rwd.model.DmpMessage;
import com.rabbitmq.client.Channel;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.core.*;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import java.io.IOException;

/**
 * @author: yaoll
 * @date: 2020-10-22
 * @verison: 1.0
 */
@Slf4j
@Configuration
public class JmsConfig {


	@Value("${persagy.dmp.exchange}")
	private String exchange;

	@Value("${persagy.dmp.alarm.routingKey}")
	private String dmpAlarmRoutingKey;

	@Value("${alarm.obj.routingKey}")
	private String alarmObjRoutingKey;

	@Value("${persagy.dmp.rwd.routingKey}")
	private String rwdRoutingKey;

	@Value("${persagy.dmp.alarm.queue}")
	private String alarmQueue;

	/**
	 * 报警对象队列
	 */
	private final String alarmObjQueue = "alarm-obj-queue";

	@Autowired
	private MessageProcesser messageProcesser;

	@Bean
	public TopicExchange exchange() {
		return new TopicExchange(exchange);
	}

	@Bean
	public Queue alarmQueue() {
		return new Queue(alarmQueue, true);
	}

	@Bean
	public Binding alarmBinding() {
		return BindingBuilder.bind(alarmQueue()).to(exchange()).with(dmpAlarmRoutingKey);
	}

	@Bean
	public Binding rwdBinding() {
		return BindingBuilder.bind(alarmQueue()).to(exchange()).with(rwdRoutingKey);
	}

	@Bean
	public Queue alarmObjQueue() {
		return new Queue(alarmObjQueue, true);
	}

	@Bean
	public Binding alarmObjBinding() {
		return BindingBuilder.bind(alarmObjQueue()).to(exchange()).with(alarmObjRoutingKey);
	}

	@RabbitListener(queues = {alarmObjQueue, "${persagy.dmp.alarm.queue}"})    //监听器监听指定的Queue
	public void process(String message, Channel channel, Message msg) {
		log.info("============================== Receive:" + message);
		DmpMessage dmpMessage = JacksonMapper.toObject(message, DmpMessage.class);
		messageProcesser.listen(dmpMessage);
		// 手动确认消息已消费
		try {
			channel.basicAck(msg.getMessageProperties().getDeliveryTag(),false);
		} catch (IOException e) {
			log.error("消息消费反馈失败", e);
		}
	}

//	@RabbitListener(queues = "${persagy.dmp.alarm.queue}")    //监听器监听指定的Queue
//	public void process(DmpMessage message) {
//		log.info("============================== Receive:" + JacksonMapper.toSimpleJson(message));
//		messageProcesser.listen(message);
//	}

}
